import datetime
import json
from six import string_types

from django.http import HttpResponse, HttpResponseRedirect
from django.core.urlresolvers import reverse, reverse_lazy, resolve
from django.core import serializers
from django.conf import settings
from django.contrib import messages
from django.utils.translation import ugettext as _

from django.db.models import Count, Avg
from django.core.files import File
from django.shortcuts import get_object_or_404

from instances.views import InstanceFormMixin, InstanceViewMixin

from speeches.aggregates import Length
from speeches.forms import SpeechForm, SpeechAudioForm, SectionForm, RecordingAPIForm, SpeakerForm, SectionPickForm, RecordingForm, RecordingTimestampFormSet
from speeches.models import Speech, Speaker, Section, Recording, Tag, RecordingTimestamp
import speeches.utils
from speeches.utils import AudioHelper, AudioException
from speeches.mixins import Base32SingleObjectMixin, UnmatchingSlugException

from django.views.generic import View, CreateView, UpdateView, DeleteView, DetailView, ListView, RedirectView, FormView
from django.views.generic.detail import SingleObjectMixin
from django.views.generic.edit import BaseFormView

from django_select2.views import AutoResponseView

import logging

logger = logging.getLogger(__name__)

class AddAnSRedirectView(RedirectView):
    url = '/%(path)s%(suffix)s'
    permanent = True
    query_string = True
    suffix = 's'

    def get_redirect_url(self, **kwargs):
        kwargs['suffix'] = self.suffix
        return super(AddAnSRedirectView, self).get_redirect_url(**kwargs)

class JSONResponseMixin(object):
    """Mixin for returning HTTPResponse of JSON data"""

    def render_to_response(self, context, **kwargs):
        kwargs['content_type'] = 'application/json'
        if not isinstance(context, string_types):
            context = json.dumps(context)
        location = kwargs.pop('location', None)
        response = HttpResponse(context, **kwargs)
        if location:
            response['Location'] = location
        return response

class NamespaceMixin(object):
    """Mixin for adding current_app based on namespace"""

    def reverse(self, template, **kwargs):
        kwargs['current_app'] = resolve(self.request.path).namespace
        return reverse(template, **kwargs)

    def reverse_lazy(self, template, **kwargs):
        kwargs['current_app'] = resolve(self.request.path).namespace
        return reverse_lazy(template, **kwargs)

    def render_to_response(self, context, **kwargs):
        kwargs['current_app'] = resolve(self.request.path).namespace
        return super(NamespaceMixin, self).render_to_response(context, **kwargs)

class SpeechAudioCreate(JSONResponseMixin, BaseFormView):
    form_class = SpeechAudioForm
    http_method_names = ['post']

    def form_valid(self, form):
        # The cleaned_data contains the TemporaryUploadedFile (File/UploadedFile subclass).
        # The form.instance contains the FieldFile with the magic save() we want.
        audio = form.cleaned_data['audio']
        form.instance.audio.save(audio.name, audio, save=False)
        return self.render_to_response({ 'status': 'done', 'filename': form.instance.audio.name })

    def form_invalid(self, form):
        return self.render_to_response({ 'error': form.errors['audio'] })

class SpeechMixin(NamespaceMixin, InstanceFormMixin):
    model = Speech
    form_class = SpeechForm

    def get_form(self, form_class):
        form = super(SpeechMixin, self).get_form(form_class)

        form.fields['section'].queryset = Section.objects.for_instance(self.request.instance)
        form.fields['speaker'].queryset = Speaker.objects.for_instance(self.request.instance)
        form.fields['tags'].queryset = Tag.objects.for_instance(self.request.instance)

        # Instance in the sense of https://github.com/mysociety/django-subdomain-instances
        form.fields['speaker'].instance = self.request.instance
        form.fields['section'].instance = self.request.instance

        return form

class SpeechDelete(SpeechMixin, DeleteView):

    def get_success_url(self):
        return self.reverse('speeches:parentless-list')

class SpeechCreate(SpeechMixin, CreateView):
    def get_context_data(self, **kwargs):
        context = super(SpeechCreate, self).get_context_data(**kwargs)
        added = self.request.GET.get('added', None)
        section = self.request.GET.get('section', None)
        if added and section:
            section_object = Section.objects.get(pk=section)
            if section_object:
                context['added'] = added
                context['section'] = section_object
        return context

    def get_initial(self):
        initial = super(SpeechCreate, self).get_initial()
        initial = initial.copy()
        try:
            speaker = int(self.request.GET['speaker'])
        except:
            speaker = None
        if speaker:
            try:
                initial['speaker'] = Speaker.objects.get(pk=speaker)
            except Speaker.DoesNotExist:
                # Ignore the supplied speaker
                pass
        try:
            section = int(self.request.GET['section'])
        except:
            section = None
        if section:
            try:
                section_object = Section.objects.get(pk=section)
                initial['section'] = section_object

                # default things to last speech in section
                try:
                    speech = section_object.speech_set.reverse()[0]

                    # Basic data is just defaulted from the last speech
                    initial['location'] = speech.location
                    initial['heading'] = speech.heading
                    initial['event'] = speech.event
                    initial['public'] = speech.public
                    initial['source_url'] = speech.source_url

                    # For speaker, to make it simpler to transcribe a dialogue between A -> B -> A -> B
                    # we default to the speaker of the *penultimate* speech, if that exists, or latest one
                    # otherwise.
                    if not speaker:
                        try:
                            penultimate_speech = section_object.speech_set.reverse()[1]
                            initial['speaker'] = penultimate_speech.speaker
                        except IndexError:
                            initial['speaker'] = speech.speaker

                    # We attempt to do vaguely clever things with the end or start time of previous speech.
                    if speech.end_date:
                        speech_end_datetime = datetime.datetime.combine(
                                speech.end_date, speech.end_time or datetime.time(0,0))
                        if speech.end_time:
                            if speech.start_date and speech.start_time:
                                speech_start_datetime = datetime.datetime.combine(speech.start_date, speech.start_time)
                                if speech_start_datetime == speech_end_datetime:
                                    speech_end_datetime = speech_end_datetime + datetime.timedelta(seconds=1)
                            initial['start_time'] = speech_end_datetime.time()
                        initial['start_date'] = speech_end_datetime.date()
                    elif speech.start_date:
                        if speech.start_time:
                            speech_start_datetime = datetime.datetime.combine(speech.start_date, speech.start_time)
                            speech_start_datetime = speech_start_datetime + datetime.timedelta(seconds=10)
                            initial['start_time'] = speech_start_datetime.time()
                            initial['start_date'] = speech_start_datetime.date()
                        else:
                            initial['start_date'] = speech.start_date

                except IndexError:
                    # don't attempt to default anything
                    pass

            except Section.DoesNotExist:
                # Ignore the supplied section
                pass
        return initial

    def get_success_url(self):
        return "{}?created".format(self.object.get_absolute_url())

    def form_valid(self, form):
        resp = super(SpeechCreate, self).form_valid(form)

        # Now set off a Celery task to transcribe the audio for this speech
        self.object.start_transcribing()

        if 'add_another' in self.request.POST:
            url = self.reverse('speeches:speech-add')
            speech = self.object
            if speech.section_id:
                url = url + '?section=%d&added=%d' % (speech.section_id, speech.id)
            return HttpResponseRedirect( url )
        else:
            return resp

class SpeechUpdate(SpeechMixin, UpdateView):
    pass

class SpeechView(NamespaceMixin, InstanceViewMixin, DetailView):
    model = Speech

    def get_queryset(self):
        return super(SpeechView, self).get_queryset().visible(self.request)

class InstanceView(NamespaceMixin, InstanceViewMixin, ListView):
    """Done as a ListView on Speech to get recent speeches, we get instance for
    free in the request."""
    model = Speech
    paginate_by = 20

    # Use a slightly different template
    def get_template_names(self):
        return [
            "speeches/%s/home.html" % self.request.instance.label,
            "speeches/home.html"
        ]

    def get_context_data(self, **kwargs):
        context = super(InstanceView, self).get_context_data(**kwargs)
        context['count_speeches'] = Speech.objects.for_instance(self.request.instance).visible(self.request).count()
        context['count_sections'] = Section.objects.for_instance(self.request.instance).count()
        context['count_speakers'] = Speaker.objects.for_instance(self.request.instance).count()
        context['average_length'] = Speech.objects.for_instance(self.request.instance).annotate(length=Length('text')).aggregate(avg=Avg('length'))['avg']
        return context

# It doesn't actually use base32 IDs in the URL, but this works around Django
# 1.4 generic view bug, and allows non-canonical slug redirects to Just Work.
class SpeakerView(NamespaceMixin, InstanceViewMixin, Base32SingleObjectMixin, ListView):
    model = Speaker
    paginate_by = 50
    template_name = 'speeches/speaker_detail.html'
    slug_field = 'slugs__slug'

    def get_queryset(self):
        queryset = super(SpeakerView, self).get_queryset()
        self.object = self.get_object(queryset)
        return self.object.speech_set.all().visible(self.request).select_related('section', 'speaker').prefetch_related('tags')

    def get_context_data(self, **kwargs):
        kwargs['speech_list'] = self.object_list
        context = super(SpeakerView, self).get_context_data(**kwargs)
        context['section_count'] = self.object.speech_set.all().visible(self.request).aggregate(Count('section', distinct=True))['section__count']
        context['longest_speech'] = self.object.speech_set.annotate(length=Length('text')).order_by('-length')[:1]
        return context

class SpeakerMixin(NamespaceMixin, InstanceFormMixin):
    model = Speaker
    form_class = SpeakerForm

class SpeakerList(NamespaceMixin, InstanceViewMixin, ListView):
    model = Speaker
    context_object_name = 'speaker_list'

class SpeakerCreate(SpeakerMixin, CreateView):
    pass

class SpeakerUpdate(SpeakerMixin, UpdateView):
    pass

class ParentlessList(NamespaceMixin, InstanceViewMixin, ListView):
    model = Speech
    template_name = 'speeches/parentless_list.html'
    paginate_by = 50

    def get_queryset(self):
        qs = super(ParentlessList, self).get_queryset()
        qs = qs.visible(self.request)
        qs = qs.filter(section=None)
        qs = qs.select_related('speaker').prefetch_related('tags')
        return qs

    def get_context_data(self, **kwargs):
        context = super(ParentlessList, self).get_context_data(**kwargs)

        # Add in a QuerySet of all the sections without a parent
        context['section_list'] = Section.objects.for_instance(self.request.instance).filter(parent=None)
        return context

class SectionMixin(NamespaceMixin, InstanceFormMixin):
    model = Section
    form_class = SectionForm

    def get_form(self, form_class):
        form = super(SectionMixin, self).get_form(form_class)
        form.fields['parent'].queryset = form.fields['parent'].queryset.filter(instance=self.request.instance)
        return form

class SectionCreate(SectionMixin, CreateView):
    def get_initial(self):
        initial = super(SectionCreate, self).get_initial()
        initial = initial.copy()
        try:
            section = int(self.request.GET['section'])
        except:
            section = None
        if section:
            try:
                initial['parent'] = Section.objects.get(pk=section)
            except Section.DoesNotExist:
                # Ignore the supplied section
                pass
        return initial

class SectionUpdate(SectionMixin, UpdateView):
    pass

class SectionDelete(SectionMixin, DeleteView):
    def get_success_url(self):
        return self.reverse('speeches:parentless-list')

class SectionView(NamespaceMixin, InstanceViewMixin, DetailView):
    model = Section

    def get(self, request, *args, **kwargs):
        try:
            return super(SectionView, self).get(request, *args, **kwargs)
        except UnmatchingSlugException as e:
            return HttpResponseRedirect(e.args[0])

    def get_object(self, queryset=None):
        pk = self.kwargs.get(self.pk_url_kwarg, None)
        if pk is not None:
            return super(SectionView, self).get_object(queryset)
        full_slug = self.kwargs.get('full_slug', None)
        slugs = full_slug.split('/')
        parent = None
        for i, slug in enumerate(slugs):
            obj = get_object_or_404(self.model, instance=self.request.instance, slugs__slug=slug, parent=parent)
            if slug != obj.slug:
                new_url = obj.get_absolute_url()
                if i < len(slugs) - 1:
                    new_url += '/' + '/'.join(slugs[i+1:])
                raise UnmatchingSlugException(new_url)
            parent = obj
        return obj

    def get_context_data(self, **kwargs):
        all_speeches = kwargs.pop('all_speeches', False)
        # Call the base implementation first to get a context
        context = super(SectionView, self).get_context_data(**kwargs)
        # Add in a QuerySet of all the speeches in this section
        context['section_tree'] = kwargs['object'].get_descendants_tree_with_speeches(
            self.request, all_speeches = all_speeches
        )
        return context

class SectionViewAN(SectionView):
    template_name = 'speeches/section_detail.an'

    def render_to_response(self, context, **response_kwargs):
        response_kwargs['content_type'] = 'text/xml'
        return super(SectionView, self).render_to_response(context, **response_kwargs)

    def get_context_data(self, **kwargs):
        kwargs['all_speeches'] = True
        context = super(SectionViewAN, self).get_context_data(**kwargs)
        speakers = set(
            s[0].speaker
                for s in context['section_tree']
                if isinstance(s[0], Speech) and s[0].speaker
        )
        context['speakers'] = speakers
        context['server_name'] = self.request.META.get('SERVER_NAME')
        return context

class BothObjectAndFormMixin(object):
    def get_context_data(self, **kwargs):
        context = super(BothObjectAndFormMixin, self).get_context_data(**kwargs)
        if 'object' not in context:
            context['object'] = self.get_object()
        if 'form' not in context:
            context['form'] = SectionPickForm()
        # Restrict to request's instance (can we think of a nicer way to do this?)
        context['form'].fields['section'].queryset = Section.objects.for_instance(self.request.instance)
        return context

class RecordingList(NamespaceMixin, InstanceViewMixin, ListView):
    model = Recording

class RecordingDisplay(NamespaceMixin, BothObjectAndFormMixin, InstanceViewMixin, DetailView):
    model = Recording

class RecordingSetSection(NamespaceMixin, BothObjectAndFormMixin, InstanceFormMixin, FormView, SingleObjectMixin):
    template_name = 'speeches/recording_detail.html'
    form_class = SectionPickForm
    model = Recording

    def get_success_url(self):
        return self.object.get_absolute_url()

    def form_valid(self, form):
        self.object = self.get_object()
        num = self.object.add_speeches_to_section(form.cleaned_data['section'])
        messages.add_message(self.request, messages.SUCCESS, _("Speeches assigned."))
        return super(RecordingSetSection, self).form_valid(form)

class RecordingView(NamespaceMixin, View):
    def get(self, request, *args, **kwargs):
        view = RecordingDisplay.as_view()
        return view(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        view = RecordingSetSection.as_view()
        return view(request, *args, **kwargs)

# NB: this is not an UpdateView, at least for now, as it only updates
# RecordingTimestamp not the actual Recording class.
class RecordingUpdate(NamespaceMixin, InstanceFormMixin, DetailView):
    model = Recording
    template_name_suffix = "_form"

    def get_context_data(self, **kwargs):
        context = super(RecordingUpdate, self).get_context_data(**kwargs)
        context['recordingtimestamp_formset'] = RecordingTimestampFormSet(self.request.POST or None, instance=self.object)
        return context

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        context = self.get_context_data(object=self.object) # Sigh
        recordingtimestamp_formset = context['recordingtimestamp_formset']

        recording = self.object

        for form in recordingtimestamp_formset:
            # in case we are processing any extra fields, which don't have
            # this set already
            form.fields['timestamp'].recording_start = recording.start_datetime

        if recordingtimestamp_formset.is_valid():
            recordingtimestamp_formset.save()

            recording.create_or_update_speeches(self.request.instance)

            # then delete the associated speeches, as Django can't
            # infer a cascade here
            for form in recordingtimestamp_formset.deleted_forms:
                try:
                    form.instance.speech.delete()
                except:
                    logger.info("Timestamp isn't linked to speech")

            return HttpResponseRedirect( recording.get_absolute_url() )
        return self.render_to_response(context)

class RecordingAPICreate(InstanceFormMixin, JSONResponseMixin, CreateView):
    # View for RecordingAPIForm, to create a recording
    model = Recording
    form_class = RecordingAPIForm

    # Limit this view to POST requests, we don't want to show an HTML form for it
    http_method_names = ['post']

    success_url = 'DUMMY'

    def get_form_kwargs(self):
        kwargs = super(RecordingAPICreate, self).get_form_kwargs()
        kwargs['request'] = self.request
        return kwargs

    def form_valid(self, form):
        logger.info("Processing recording")
        logger.info("  " + repr(form.cleaned_data))

        super(RecordingAPICreate, self).form_valid(form)

        recording = self.object

        recording_timestamps = form.cleaned_data.get('timestamps', [])
        for recording_timestamp in recording_timestamps:
            recording_timestamp.recording = recording
            recording_timestamp.save()

        if len(recording_timestamps):
            recording.start_datetime = recording_timestamps[0].timestamp
            recording.save()

        # Create speeches from the recording
        speeches = recording.create_or_update_speeches(self.request.instance)

        # Transcribe each speech
        for speech in speeches:
            speech.start_transcribing()

        # Return a 201 response
        serialisable_fields = ('audio', 'timestamps')
        serialised = serializers.serialize("json", [recording], fields=serialisable_fields)
        serialised = serialised[1:-1]
        return self.render_to_response(serialised, status=201, location=reverse("speeches:recording-view", args=[recording.id]))

    def form_invalid(self, form):
        return self.render_to_response({ 'errors': json.dumps(form.errors) }, status=400)


class Select2AutoResponseView(AutoResponseView):
    def check_all_permissions(self, request, *args, **kwargs):
        super(Select2AutoResponseView, self).check_all_permissions(request, *args, **kwargs)

        model = request._AutoResponseView__django_select2_local.model
        request._AutoResponseView__django_select2_local.queryset = model.objects.for_instance(request.instance)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.db.models.deletion
import speeches.models
import easy_thumbnails.fields
import sluggable.fields


class Migration(migrations.Migration):

    dependencies = [
        ('instances', '__first__'),
        ('popolo', '__first__'),
        ('contenttypes', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Recording',
            fields=[
                ('id', models.AutoField(serialize=False, primary_key=True, auto_created=True, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('modified', models.DateTimeField(auto_now=True)),
                ('audio', models.FileField(max_length=255, upload_to='recordings/%Y-%m-%d/')),
                ('start_datetime', models.DateTimeField(help_text='Datetime of first timestamp associated with recording', null=True, blank=True)),
                ('audio_duration', models.IntegerField(help_text='Duration of recording, in seconds', blank=True, default=0)),
                ('instance', models.ForeignKey(to='instances.Instance')),
            ],
            options={
                'abstract': False,
            },
            bases=(speeches.models.AudioMP3Mixin, models.Model),
        ),
        migrations.CreateModel(
            name='RecordingTimestamp',
            fields=[
                ('id', models.AutoField(serialize=False, primary_key=True, auto_created=True, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('modified', models.DateTimeField(auto_now=True)),
                ('timestamp', models.DateTimeField(db_index=True)),
                ('instance', models.ForeignKey(to='instances.Instance')),
                ('recording', models.ForeignKey(related_name='timestamps', to='speeches.Recording', default=0)),
            ],
            options={
                'ordering': ('timestamp',),
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Section',
            fields=[
                ('id', models.AutoField(serialize=False, primary_key=True, auto_created=True, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('modified', models.DateTimeField(auto_now=True)),
                ('num', models.TextField(help_text='The number of the section', verbose_name='number', blank=True)),
                ('heading', models.TextField(help_text='The heading of the section', verbose_name='heading', blank=True)),
                ('subheading', models.TextField(help_text='The subheading of the section', verbose_name='subheading', blank=True)),
                ('description', models.TextField(help_text='Longer description, HTML', verbose_name='description', blank=True)),
                ('start_date', models.DateField(help_text='What date did the section start?', null=True, verbose_name='start date', blank=True)),
                ('start_time', models.TimeField(help_text='What time did the section start?', null=True, verbose_name='start time', blank=True)),
                ('number', models.TextField(verbose_name='document number', blank=True)),
                ('legislature', models.TextField(verbose_name='legislature', blank=True)),
                ('session', models.TextField(help_text='Legislative session', verbose_name='session', blank=True)),
                ('slug', sluggable.fields.SluggableField(verbose_name='slug')),
                ('source_url', models.TextField(verbose_name='source URL', blank=True)),
                ('instance', models.ForeignKey(to='instances.Instance')),
                ('parent', models.ForeignKey(related_name='children', to='speeches.Section', null=True, blank=True, verbose_name='parent')),
            ],
            options={
                'ordering': ('id',),
                'verbose_name': 'section',
                'verbose_name_plural': 'sections',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Slug',
            fields=[
                ('id', models.AutoField(serialize=False, primary_key=True, auto_created=True, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField()),
                ('slug', models.CharField(max_length=255, verbose_name='URL', db_index=True)),
                ('redirect', models.BooleanField(default=False, verbose_name='Redirection')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('content_type', models.ForeignKey(to='contenttypes.ContentType')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Speaker',
            fields=[
                ('person_ptr', models.OneToOneField(to='popolo.Person', auto_created=True, parent_link=True, serialize=False, primary_key=True)),
                ('slug', sluggable.fields.SluggableField(verbose_name='slug')),
                ('image_cache', easy_thumbnails.fields.ThumbnailerImageField(help_text='If image is set, a local copy will be stored here.', null=True, upload_to=speeches.models.upload_to, verbose_name='image_cache', blank=True)),
                ('instance', models.ForeignKey(to='instances.Instance')),
            ],
            options={
                'ordering': ('name',),
                'verbose_name': 'speaker',
                'verbose_name_plural': 'speakers',
            },
            bases=('popolo.person', models.Model),
        ),
        migrations.CreateModel(
            name='Speech',
            fields=[
                ('id', models.AutoField(serialize=False, primary_key=True, auto_created=True, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('modified', models.DateTimeField(auto_now=True)),
                ('audio', models.FileField(max_length=255, verbose_name='audio', blank=True, upload_to='speeches/%Y-%m-%d/')),
                ('text', models.TextField(help_text='The text of the speech', verbose_name='text', blank=True)),
                ('num', models.TextField(help_text='The number of the speech, if relevant', verbose_name='number', blank=True)),
                ('heading', models.TextField(help_text='The heading of the speech, if relevant', verbose_name='heading', blank=True)),
                ('subheading', models.TextField(help_text='The subheading of the speech, if relevant', verbose_name='subheading', blank=True)),
                ('event', models.TextField(help_text='Was the speech at a particular event?', db_index=True, verbose_name='event', blank=True)),
                ('location', models.TextField(help_text='Where the speech took place', db_index=True, verbose_name='location', blank=True)),
                ('speaker_display', models.CharField(null=True, max_length=256, blank=True)),
                ('type', models.CharField(help_text='What sort of speech is this?', max_length=32, verbose_name='type', choices=[('speech', 'Speech'), ('question', 'Question'), ('answer', 'Answer'), ('scene', 'Scene'), ('narrative', 'Narrative'), ('summary', 'Summary'), ('other', 'Other')])),
                ('start_date', models.DateField(help_text='What date did the speech start?', null=True, verbose_name='start date', blank=True)),
                ('start_time', models.TimeField(help_text='What time did the speech start?', null=True, verbose_name='start time', blank=True)),
                ('end_date', models.DateField(help_text='What date did the speech end?', null=True, verbose_name='end date', blank=True)),
                ('end_time', models.TimeField(help_text='What time did the speech end?', null=True, verbose_name='end time', blank=True)),
                ('public', models.BooleanField(help_text='Is this speech public?', verbose_name='public', default=True)),
                ('source_url', models.TextField(verbose_name='source URL', blank=True)),
                ('celery_task_id', models.CharField(null=True, max_length=256, verbose_name='celery task ID', blank=True)),
                ('instance', models.ForeignKey(to='instances.Instance')),
                ('section', models.ForeignKey(to='speeches.Section', help_text='The section that this speech is contained in', null=True, blank=True, on_delete=django.db.models.deletion.SET_NULL, verbose_name='Section')),
                ('speaker', models.ForeignKey(to='speeches.Speaker', help_text='Who gave this speech?', null=True, blank=True, on_delete=django.db.models.deletion.SET_NULL, verbose_name='speaker')),
            ],
            options={
                'ordering': ('start_date', 'start_time', 'id'),
                'verbose_name': 'speech',
                'verbose_name_plural': 'speeches',
            },
            bases=(speeches.models.AudioMP3Mixin, models.Model),
        ),
        migrations.CreateModel(
            name='Tag',
            fields=[
                ('id', models.AutoField(serialize=False, primary_key=True, auto_created=True, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('modified', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(unique=True, max_length=100)),
                ('instance', models.ForeignKey(to='instances.Instance')),
            ],
            options={
                'verbose_name': 'tag',
                'verbose_name_plural': 'tags',
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='speech',
            name='tags',
            field=models.ManyToManyField(to='speeches.Tag', null=True, verbose_name='tags', blank=True),
            preserve_default=True,
        ),
        migrations.AlterUniqueTogether(
            name='speaker',
            unique_together=set([('instance', 'slug')]),
        ),
        migrations.AlterUniqueTogether(
            name='section',
            unique_together=set([('parent', 'slug', 'instance')]),
        ),
        migrations.AddField(
            model_name='recordingtimestamp',
            name='speaker',
            field=models.ForeignKey(to='speeches.Speaker', null=True, on_delete=django.db.models.deletion.SET_NULL, blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='recordingtimestamp',
            name='speech',
            field=models.ForeignKey(to='speeches.Speech', null=True, on_delete=django.db.models.deletion.SET_NULL, blank=True),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    depends_on = (
        ("instances", "0001_initial"),
    )
    needed_by = (
        ("instances", "0002_auto__add_field_instance_description"),
    )

    def forwards(self, orm):
        instance = orm['instances.Instance'](label='default')
        instance.save()

        # Adding field 'Meeting.instance'
        db.add_column('speeches_meeting', 'instance',
                      self.gf('django.db.models.fields.related.ForeignKey')(default=instance.id, to=orm['instances.Instance']),
                      keep_default=False)

        # Adding field 'Debate.instance'
        db.add_column('speeches_debate', 'instance',
                      self.gf('django.db.models.fields.related.ForeignKey')(default=instance.id, to=orm['instances.Instance']),
                      keep_default=False)

        # Adding field 'RecordingTimestamp.instance'
        db.add_column('speeches_recordingtimestamp', 'instance',
                      self.gf('django.db.models.fields.related.ForeignKey')(default=instance.id, to=orm['instances.Instance']),
                      keep_default=False)

        # Adding field 'Speech.instance'
        db.add_column('speeches_speech', 'instance',
                      self.gf('django.db.models.fields.related.ForeignKey')(default=instance.id, to=orm['instances.Instance']),
                      keep_default=False)

        # Adding field 'Speaker.instance'
        db.add_column('speeches_speaker', 'instance',
                      self.gf('django.db.models.fields.related.ForeignKey')(default=instance.id, to=orm['instances.Instance']),
                      keep_default=False)

        # Adding field 'Recording.instance'
        db.add_column('speeches_recording', 'instance',
                      self.gf('django.db.models.fields.related.ForeignKey')(default=instance.id, to=orm['instances.Instance']),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'Meeting.instance'
        db.delete_column('speeches_meeting', 'instance_id')

        # Deleting field 'Debate.instance'
        db.delete_column('speeches_debate', 'instance_id')

        # Deleting field 'RecordingTimestamp.instance'
        db.delete_column('speeches_recordingtimestamp', 'instance_id')

        # Deleting field 'Speech.instance'
        db.delete_column('speeches_speech', 'instance_id')

        # Deleting field 'Speaker.instance'
        db.delete_column('speeches_speaker', 'instance_id')

        # Deleting field 'Recording.instance'
        db.delete_column('speeches_recording', 'instance_id')

        orm['instances.Instance'].objects.get(label='default').delete()

    models = {
        'instances.instance': {
            'Meta': {'object_name': 'Instance'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('instances.fields.DNSLabelField', [], {'unique': 'True', 'max_length': '63', 'db_index': 'True'})
        },
        'speeches.debate': {
            'Meta': {'object_name': 'Debate'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'instance': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['instances.Instance']"}),
            'meeting': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['speeches.Meeting']", 'null': 'True', 'blank': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'speeches.meeting': {
            'Meta': {'object_name': 'Meeting'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'instance': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['instances.Instance']"}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'speeches.recording': {
            'Meta': {'object_name': 'Recording'},
            'audio': ('django.db.models.fields.files.FileField', [], {'max_length': '255'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'instance': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['instances.Instance']"}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'timestamps': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'to': "orm['speeches.RecordingTimestamp']", 'null': 'True', 'blank': 'True'})
        },
        'speeches.recordingtimestamp': {
            'Meta': {'object_name': 'RecordingTimestamp'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'instance': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['instances.Instance']"}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'speaker': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['speeches.Speaker']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'}),
            'timestamp': ('django.db.models.fields.DateTimeField', [], {'db_index': 'True'})
        },
        'speeches.speaker': {
            'Meta': {'object_name': 'Speaker'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'instance': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['instances.Instance']"}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.TextField', [], {'db_index': 'True'}),
            'popit_url': ('django.db.models.fields.TextField', [], {'unique': 'True'})
        },
        'speeches.speech': {
            'Meta': {'object_name': 'Speech'},
            'audio': ('django.db.models.fields.files.FileField', [], {'max_length': '255', 'blank': 'True'}),
            'celery_task_id': ('django.db.models.fields.CharField', [], {'max_length': '256', 'null': 'True', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'debate': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['speeches.Debate']", 'null': 'True', 'blank': 'True'}),
            'end_date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'end_time': ('django.db.models.fields.TimeField', [], {'null': 'True', 'blank': 'True'}),
            'event': ('django.db.models.fields.TextField', [], {'db_index': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'instance': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['instances.Instance']"}),
            'location': ('django.db.models.fields.TextField', [], {'db_index': 'True', 'blank': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'source_url': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'speaker': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['speeches.Speaker']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'}),
            'start_date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'start_time': ('django.db.models.fields.TimeField', [], {'null': 'True', 'blank': 'True'}),
            'text': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'title': ('django.db.models.fields.TextField', [], {'blank': 'True'})
        }
    }

    complete_apps = ['speeches']



# -*- coding: utf-8 -*-
from south.v2 import DataMigration
from django.utils.encoding import smart_text

# Note: Don't use "from appname.models import ModelName".
# Use orm.ModelName to refer to models in this application,
# and orm['appname.ModelName'] for models in other applications.

import functools
import types

# From http://south.aeracode.org/ticket/325
def get_content_type(orm, app_label, model_name):
    # sometimes these content types are not yet created when this migration runs,
    # so create them here.  mimic behavior of ContentType.objects.get_for_model
    # (which is not available in migrations)
    opts = orm['%s.%s' % (app_label, model_name)]._meta.concrete_model._meta
    ct, created = orm['contenttypes.ContentType'].objects.get_or_create(app_label=app_label,
        model=model_name, defaults={'name': smart_text(opts.verbose_name_raw)})
    return ct

# This function from django-sluggable is needed because the post-delete signal
# on SluggableField (to delete all the attached slugs) calls it
def filter_by_obj(self, obj, **kwargs):
    content_type = kwargs.pop('content_type')
    return self._filter_or_exclude(False, content_type_id=content_type.pk, object_id=obj.pk, **kwargs)

def attach_filter_by_obj(obj, **kwargs):
    obj.filter_by_obj = types.MethodType(functools.partial(filter_by_obj, **kwargs), obj)

class Migration(DataMigration):
    no_dry_run = True

    def forwards(self, orm):
        old_ct = get_content_type(orm, 'speeches', 'speaker')
        new_ct = get_content_type(orm, 'speeches', 'popolospeaker')
        popolo_ct = get_content_type(orm, 'popolo', 'person')

        try:
            an_ai = orm['popit.ApiInstance'].objects.get(url='http://import.example.org/')
        except orm['popit.ApiInstance'].DoesNotExist:
            an_ai = None

        for speaker in orm.Speaker.objects.all():
            # Create new speaker
            new_speaker = orm.PopoloSpeaker(
                name=speaker.name,
                slug=speaker.slug,
                instance=speaker.instance,
            )
            new_speaker.slug_changed = False
            if speaker.person:
                if not new_speaker.name:
                    new_speaker.name = speaker.person.name
                new_speaker.image = speaker.person.image
                new_speaker.summary = speaker.person.summary
            new_speaker.save()

            # Transfer slugs across
            speaker.slugs = orm.Slug.objects.filter(content_type=old_ct, object_id=speaker.id)
            attach_filter_by_obj(speaker.slugs, content_type=old_ct)
            for slug in speaker.slugs:
                slug.content_type = new_ct
                slug.object_id = new_speaker.id
                slug.save()

            # Make sure IDs are transferred appropriately
            if speaker.person:
                if speaker.person.popit_id:
                    scheme = 'PopIt ID'
                    if speaker.person.api_instance == an_ai:
                        scheme = 'Akoma Ntoso import'
                    orm['popolo.Identifier'].objects.create(
                        content_type=popolo_ct,
                        object_id=new_speaker.id,
                        identifier=speaker.person.popit_id,
                        scheme=scheme,
                    )
                if speaker.person.popit_url:
                    orm['popolo.Identifier'].objects.create(
                        content_type=popolo_ct,
                        object_id=new_speaker.id,
                        identifier=speaker.person.popit_url,
                        scheme='PopIt URL',
                    )

            for model in (orm.Speech, orm.RecordingTimestamp):
                model.objects.filter(speaker=speaker).update(speaker_new=new_speaker)

            # Remove the old speaker
            speaker.delete()

        # No speaker objects any more, so no need for any Person objects
        orm['popit.Person'].objects.all().delete()

    def backwards(self, orm):
        old_ct = get_content_type(orm, 'speeches', 'speaker')
        new_ct = get_content_type(orm, 'speeches', 'popolospeaker')
        popolo_ct = get_content_type(orm, 'popolo', 'person')

        an_ai, _ = orm['popit.ApiInstance'].objects.get_or_create(url='http://import.example.org/')
        all_ai, _ = orm['popit.ApiInstance'].objects.get_or_create(url='http://all.example.org/')

        for speaker in orm.PopoloSpeaker.objects.all():
            old_speaker = orm.Speaker(
                name=speaker.name,
                slug=speaker.slug,
                instance=speaker.instance,
            )
            old_speaker.slug_changed = False
            old_speaker.save()

            speaker.slugs = orm.Slug.objects.filter(content_type=new_ct, object_id=speaker.id)
            attach_filter_by_obj(speaker.slugs, content_type=new_ct)
            for slug in speaker.slugs:
                slug.content_type = old_ct
                slug.object_id = old_speaker.id
                slug.save()

            speaker.identifiers = orm['popolo.Identifier'].objects.filter(content_type=popolo_ct, object_id=speaker.id)
            an_imported = speaker.identifiers.filter(scheme='Akoma Ntoso import')
            popit_url = speaker.identifiers.filter(scheme='PopIt URL')
            popit_id = speaker.identifiers.filter(scheme='PopIt ID')

            data = {
                'api_instance': all_ai,
                'name': speaker.name,
                'image': speaker.image or '',
                'summary': speaker.summary,
            }
            if an_imported or speaker.image or speaker.summary or popit_url or popit_id:
                if an_imported:
                    data['api_instance'] = an_ai
                    data['popit_id'] = an_imported[0].identifier
                else:
                    if popit_url:
                        data['popit_url'] = popit_url[0].identifier
                    if popit_id:
                        data['popit_id'] = popit_id[0].identifier
                p = orm['popit.Person'].objects.create(**data)
                old_speaker.person = p
                old_speaker.save()

            for model in (orm.Speech, orm.RecordingTimestamp):
                model.objects.filter(speaker_new=speaker).update(speaker=old_speaker)

            speaker.delete()

        assert orm.PopoloSpeaker.objects.count() == 0, "PopoloSpeaker objects still exist"

    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'instances.instance': {
            'Meta': {'object_name': 'Instance'},
            'created_by': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'created_instances'", 'null': 'True', 'to': "orm['auth.User']"}),
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('instances.fields.DNSLabelField', [], {'unique': 'True', 'max_length': '63', 'db_index': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'users': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'instances'", 'blank': 'True', 'to': "orm['auth.User']"})
        },
        'popit.apiinstance': {
            'Meta': {'object_name': 'ApiInstance'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'url': ('popit.fields.ApiInstanceURLField', [], {'unique': 'True', 'max_length': '200'})
        },
        'popit.person': {
            'Meta': {'object_name': 'Person'},
            'api_instance': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['popit.ApiInstance']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.URLField', [], {'max_length': '200', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'popit_id': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True'}),
            'popit_url': ('popit.fields.PopItURLField', [], {'default': "''", 'max_length': '200', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'summary': ('django.db.models.fields.TextField', [], {'blank': 'True'})
        },
        'popolo.contactdetail': {
            'Meta': {'object_name': 'ContactDetail'},
            'contact_type': ('django.db.models.fields.CharField', [], {'max_length': '12'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'created_at': ('model_utils.fields.AutoCreatedField', [], {'default': 'datetime.datetime.now'}),
            'end_date': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '128', 'blank': 'True'}),
            'note': ('django.db.models.fields.CharField', [], {'max_length': '128', 'blank': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'start_date': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True', 'blank': 'True'}),
            'updated_at': ('model_utils.fields.AutoLastModifiedField', [], {'default': 'datetime.datetime.now'}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '128'})
        },
        'popolo.identifier': {
            'Meta': {'object_name': 'Identifier'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'identifier': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'scheme': ('django.db.models.fields.CharField', [], {'max_length': '128', 'blank': 'True'})
        },
        'popolo.link': {
            'Meta': {'object_name': 'Link'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'note': ('django.db.models.fields.CharField', [], {'max_length': '128', 'blank': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'url': ('django.db.models.fields.URLField', [], {'max_length': '200'})
        },
        'popolo.othername': {
            'Meta': {'object_name': 'OtherName'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'end_date': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'note': ('django.db.models.fields.CharField', [], {'max_length': '256', 'blank': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'start_date': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True', 'blank': 'True'})
        },
        'popolo.person': {
            'Meta': {'object_name': 'Person'},
            'additional_name': ('django.db.models.fields.CharField', [], {'max_length': '128', 'blank': 'True'}),
            'biography': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'birth_date': ('django.db.models.fields.CharField', [], {'max_length': '10', 'blank': 'True'}),
            'created_at': ('model_utils.fields.AutoCreatedField', [], {'default': 'datetime.datetime.now'}),
            'death_date': ('django.db.models.fields.CharField', [], {'max_length': '10', 'blank': 'True'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'null': 'True', 'blank': 'True'}),
            'end_date': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True', 'blank': 'True'}),
            'family_name': ('django.db.models.fields.CharField', [], {'max_length': '128', 'blank': 'True'}),
            'gender': ('django.db.models.fields.CharField', [], {'max_length': '128', 'blank': 'True'}),
            'given_name': ('django.db.models.fields.CharField', [], {'max_length': '128', 'blank': 'True'}),
            'honorific_prefix': ('django.db.models.fields.CharField', [], {'max_length': '128', 'blank': 'True'}),
            'honorific_suffix': ('django.db.models.fields.CharField', [], {'max_length': '128', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.URLField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'patronymic_name': ('django.db.models.fields.CharField', [], {'max_length': '128', 'blank': 'True'}),
            'sort_name': ('django.db.models.fields.CharField', [], {'max_length': '128', 'blank': 'True'}),
            'start_date': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True', 'blank': 'True'}),
            'summary': ('django.db.models.fields.CharField', [], {'max_length': '512', 'blank': 'True'}),
            'updated_at': ('model_utils.fields.AutoLastModifiedField', [], {'default': 'datetime.datetime.now'})
        },
        'popolo.source': {
            'Meta': {'object_name': 'Source'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'note': ('django.db.models.fields.CharField', [], {'max_length': '128', 'blank': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'url': ('django.db.models.fields.URLField', [], {'max_length': '200'})
        },
        'speeches.popolospeaker': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('instance', 'slug'),)", 'object_name': 'PopoloSpeaker', '_ormbases': ['popolo.Person']},
            'instance': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['instances.Instance']"}),
            'person_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['popolo.Person']", 'unique': 'True', 'primary_key': 'True'}),
            'slug': ('sluggable.fields.SluggableField', [], {'unique_with': "('instance',)", 'max_length': '50', 'populate_from': "'name'"})
        },
        'speeches.recording': {
            'Meta': {'object_name': 'Recording'},
            'audio': ('django.db.models.fields.files.FileField', [], {'max_length': '255'}),
            'audio_duration': ('django.db.models.fields.IntegerField', [], {'default': '0', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'instance': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['instances.Instance']"}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'start_datetime': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'})
        },
        'speeches.recordingtimestamp': {
            'Meta': {'ordering': "('timestamp',)", 'object_name': 'RecordingTimestamp'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'instance': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['instances.Instance']"}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'recording': ('django.db.models.fields.related.ForeignKey', [], {'default': '0', 'related_name': "'timestamps'", 'to': "orm['speeches.Recording']"}),
            'speaker': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['speeches.Speaker']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'}),
            'speaker_new': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['speeches.PopoloSpeaker']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'}),
            'speech': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['speeches.Speech']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'}),
            'timestamp': ('django.db.models.fields.DateTimeField', [], {'db_index': 'True'})
        },
        'speeches.section': {
            'Meta': {'ordering': "('id',)", 'unique_together': "(('parent', 'slug', 'instance'),)", 'object_name': 'Section'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'instance': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['instances.Instance']"}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'children'", 'null': 'True', 'to': "orm['speeches.Section']"}),
            'slug': ('sluggable.fields.SluggableField', [], {'unique_with': "('parent', 'instance')", 'max_length': '50', 'populate_from': "'title'"}),
            'source_url': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'title': ('django.db.models.fields.TextField', [], {'blank': 'True'})
        },
        'speeches.slug': {
            'Meta': {'object_name': 'Slug'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'redirect': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'slug': ('django.db.models.fields.CharField', [], {'max_length': '255', 'db_index': 'True'})
        },
        'speeches.speaker': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('instance', 'slug'),)", 'object_name': 'Speaker'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'instance': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['instances.Instance']"}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.TextField', [], {'db_index': 'True'}),
            'person': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['popit.Person']", 'null': 'True', 'on_delete': 'models.PROTECT', 'blank': 'True'}),
            'slug': ('sluggable.fields.SluggableField', [], {'unique_with': "('instance',)", 'max_length': '50', 'populate_from': "'name'"})
        },
        'speeches.speech': {
            'Meta': {'ordering': "('start_date', 'start_time', 'id')", 'object_name': 'Speech'},
            'audio': ('django.db.models.fields.files.FileField', [], {'max_length': '255', 'blank': 'True'}),
            'celery_task_id': ('django.db.models.fields.CharField', [], {'max_length': '256', 'null': 'True', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'end_date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'end_time': ('django.db.models.fields.TimeField', [], {'null': 'True', 'blank': 'True'}),
            'event': ('django.db.models.fields.TextField', [], {'db_index': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'instance': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['instances.Instance']"}),
            'location': ('django.db.models.fields.TextField', [], {'db_index': 'True', 'blank': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'public': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'section': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['speeches.Section']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'}),
            'source_url': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'speaker': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['speeches.Speaker']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'}),
            'speaker_display': ('django.db.models.fields.CharField', [], {'max_length': '256', 'null': 'True', 'blank': 'True'}),
            'speaker_new': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['speeches.PopoloSpeaker']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'}),
            'start_date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'start_time': ('django.db.models.fields.TimeField', [], {'null': 'True', 'blank': 'True'}),
            'tags': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'to': "orm['speeches.Tag']", 'null': 'True', 'blank': 'True'}),
            'text': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'title': ('django.db.models.fields.TextField', [], {'blank': 'True'})
        },
        'speeches.tag': {
            'Meta': {'object_name': 'Tag'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'instance': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['instances.Instance']"}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'})
        }
    }

    complete_apps = ['speeches']
    symmetrical = True



from datetime import datetime, date, time, timedelta

from django.test import TestCase, LiveServerTestCase
from django.test.utils import override_settings
from django.contrib.auth.models import User

from selenium.webdriver.support.ui import WebDriverWait

from speeches.models import Section, Speech
from instances.models import Instance

@override_settings(
    PASSWORD_HASHERS = ( 'django.contrib.auth.hashers.MD5PasswordHasher', ),
)
class InstanceTestCase(TestCase):
    def setUp(self):
        self.instance = Instance.objects.create(label='default')
        user = User.objects.create_user(username='admin', email='admin@example.org', password='admin')
        user.instances.add(self.instance)
        self.client.login(username='admin', password='admin')

class InstanceLiveServerTestCase(LiveServerTestCase):
    def setUp(self):
        self.instance = Instance.objects.create(label='default')
        user = User.objects.create_user(username='admin', email='admin@example.org', password='admin')
        user.instances.add(self.instance)

        self.selenium.get('%s%s' % (self.live_server_url, '/accounts/login/?next=/'))
        username_input = self.selenium.find_element_by_name("username")
        username_input.send_keys('admin')
        password_input = self.selenium.find_element_by_name("password")
        password_input.send_keys('admin')
        self.selenium.find_element_by_xpath('//input[@value="Log in"]').click()

        WebDriverWait(self.selenium, 10).until(
            lambda x: x.find_element_by_link_text('Add your first statement')
        )


class ParentInstanceMismatchError(Exception):
    pass


def create_sections( subsections, parent=None, instance=None):
    """
    Create a hierachy of sections and speeches. Very useful for setting up test data as needed.

    Note - used in external packages (pombola.south_africa), alter with care.

    create_sections([
        {
            'heading': "Top level section",
            'subsections': [
                {   'heading': "Nested section",
                    'subsections': [
                        {   'heading': "Section with speeches",
                            'speeches': [ 4, date(2013, 3, 25), time(9, 0) ],
                        },
                        {   'heading': "Bill on Silly Walks",
                            'speeches': [ 2, date(2013, 3, 25), time(12, 0) ],
                        },
                    ]
                },
                {
                    'heading': "Another nested section (but completely empty)",
                    'subsections': []
                },
            ]
        },
    ])
    """

    # If given an instance and a parent check that they are compatible - we don't
    # want to end up with a hierarchy that spans instances and this would probably
    # mean that there is some confusion upstream that we should highlight.
    if instance and parent and instance != parent.instance:
        raise ParentInstanceMismatchError("The instance and parent.instance do not match")

    # If we have a parent then use the instance from that.
    if parent:
        instance = parent.instance

    # If we don't have an instance (which also means we had no parent) then create
    # one - this is very convenient in the test scripts.
    if not instance:
        instance, _ = Instance.objects.get_or_create(label='create-sections-instance')

    for subsection in subsections:
        s = Section.objects.create( instance=instance, heading=subsection['heading'], parent=parent )
        if 'subsections' in subsection:
            create_sections(subsection['subsections'], parent=s)
        if 'speeches' in subsection:
            # If there's a 4th element in speeches, that's a boolean
            # indicating whether the speeches should be public:
            speeches_details = subsection['speeches']
            if len(speeches_details) >= 4:
                public = speeches_details[3]
            else:
                public = True
            num, d, t = speeches_details[:3]
            for i in range(0, num):
                Speech.objects.create(
                    instance = instance,
                    section = s,
                    text = 'rhubarb rhubarb',
                    start_date = d,
                    start_time = t,
                    public = public,
                    source_url = "http://somewhere.or.other/{0}".format(i)
                )
                if t:
                    t = (datetime.combine(date.today(), t) + timedelta(minutes=10)).time()


class CreateSectionsTests(TestCase):
    def test_parent_instance_mismatch(self):
        foo_instance = Instance.objects.create(label="foo")
        bar_instance = Instance.objects.create(label="bar")
        foo_parent = Section.objects.create(instance=foo_instance, heading="Foo Section")

        # Should run without exception
        create_sections([], parent=foo_parent, instance=foo_instance)
        create_sections([],                    instance=foo_instance)
        create_sections([], parent=foo_parent                      )
        create_sections([],                                        )

        # Should raise
        self.assertRaises(
            ParentInstanceMismatchError,
            create_sections,
            subsections = [],
            parent = foo_parent,
            instance = bar_instance
        )



from __future__ import division

def rel_calc(n):
    """Calculate the colour-specific bit, given a colour hex-string."""
    n = int(n, 16) / 255
    if n <= 0.03928:
        n = n / 12.92
    else:
        n = ( ( n + 0.055 ) / 1.055 ) ** 2.4
    return n

def relative_luminance(h):
    """Calculate the relative luminance of a colour, given in rrggbb hex."""
    r = rel_calc(h[0:2])
    g = rel_calc(h[2:4])
    b = rel_calc(h[4:6])
    l = 0.2126 * r + 0.7152 * g + 0.0722 * b
    return l

def contrast_ratio(l1, l2):
    """Given two relative luminances, calculate their contrast ratio."""
    return ( l1 + 0.05 ) / ( l2 + 0.05 )


# Test contrast against white (L1 = 1)
# Using http://www.w3.org/TR/WCAG20/#relativeluminancedef
# l = relative_luminance(hex_colour)
# cr = contrast_ratio(1, l)
# if cr < 1.02: # Only care about the palest of pale
#     return 'cccccc'
# return hex_colour




import audioread.ffdec
import mutagen
import os
import re
import tempfile
import filecmp
import wave
from datetime import timedelta

from django.core.files import File
from django.utils import timezone

import speeches
from speeches.tests import InstanceTestCase
from speeches.utils import AudioHelper
from speeches.models import Recording, RecordingTimestamp, Speaker

def strip_kbps_from_file_info(s):
    return re.sub(r'\s+\d+\s+kbps,', '', s)

def file_info(filename):
    if filename.endswith('.mp3'):
        mp3 = mutagen.File(filename)
        return (mp3.tags.version, mp3.info.layer, mp3.info.version, mp3.info.sample_rate, mp3.info.mode)
    elif filename.endswith('.wav'):
        wav = wave.open(filename)
        return (wav.getsampwidth(), wav.getnchannels(), wav.getframerate())

class AudioHelperTests(InstanceTestCase):

    @classmethod
    def setUpClass(cls):
        cls._in_fixtures = os.path.join(os.path.abspath(speeches.__path__[0]), 'fixtures', 'test_inputs')
        cls._expected_fixtures = os.path.join(os.path.abspath(speeches.__path__[0]), 'fixtures', 'expected_outputs')

    def setUp(self):
        super(AudioHelperTests, self).setUp()
        self.helper = AudioHelper()
        self.tmp_filename = None
        self.remove_tmp_filename = False

    def tearDown(self):
        if self.tmp_filename is not None and self.remove_tmp_filename:
            os.remove(self.tmp_filename)

    def expected_output_file(self, filename):
        _, extension = os.path.splitext(filename)
        return os.path.join(self._expected_fixtures,
                            extension[1:],
                            filename)

    def assertSameAudioLength(self, filename_a, filename_b):
        files = (filename_a, filename_b)
        # Use FFMPEG directly as CoreAudio returns incorrect durations for the generated MP3s
        af1 = audioread.ffdec.FFmpegAudioFile(filename_a)
        af2 = audioread.ffdec.FFmpegAudioFile(filename_b)
        message = 'The audio files %s (%.3fs) and %s (%.3fs) were of different lengths'
        message = message % (files[0], af1.duration, files[1], af2.duration)
        # Make sure the lengths are within 0.2s of each other - this is
        # intended to be imprecise enough to ignore any differences in
        # frame padding in the MP3:
        self.assertAlmostEqual(af1.duration,
                               af2.duration,
                               delta=0.2,
                               msg=message)

    def convert(self, known_input, method, expected_output):
        self.tmp_filename = getattr(self.helper, method)(os.path.join(self._in_fixtures, known_input))
        expected = self.expected_output_file(expected_output)
        # Check that the type of the file is exactly the same first of all:
        self.assertEqual(file_info(self.tmp_filename), file_info(expected))
        # Now check that the files are identical:
        self.assertSameAudioLength(self.tmp_filename, expected)

    def test_wav_file_creation_from_mp3(self):
        self.convert('lamb.mp3', 'make_wav', 'lamb_from_mp3.wav')

    def test_wav_file_creation_from_android_3gp(self):
        self.convert('lamb.3gp', 'make_wav', 'lamb_from_3gp.wav')

    def test_wav_file_creation_from_iphone_wav(self):
        self.convert('lamb_iphone.wav', 'make_wav', 'lamb_from_iphone.wav')

    def test_wav_file_creation_from_stereo_wav(self):
        self.convert('lamb_stereo.wav', 'make_wav', 'lamb_from_stereo.wav')

    def test_mp3_file_creation_from_stereo_wav(self):
        self.convert('lamb_stereo.wav', 'make_mp3', 'lamb_mp3_from_stereo.mp3')

    def test_mp3_file_creation_from_iphone_wav(self):
        self.convert('lamb_iphone.wav', 'make_mp3', 'lamb_mp3_from_iphone.mp3')

    def test_mp3_file_creation_from_android_3gp(self):
        self.convert('lamb.3gp', 'make_mp3', 'lamb_mp3_from_3gp.mp3')

    def test_recording_splitting_no_timestamps(self):
        audio = open(os.path.join(self._in_fixtures, 'lamb.mp3'), 'rb')
        recording = Recording.objects.create(audio=File(audio, 'lamb.mp3'), instance=self.instance)

        files_created = self.helper.split_recording(recording)

        self.assertEqual(len(files_created), 1)
        self.assertSameAudioLength(files_created[0], self.expected_output_file('lamb_whole.mp3'))

    def test_recording_splitting_one_timestamp(self):
        speaker = Speaker.objects.create(name='Steve', instance=self.instance)
        timestamp = RecordingTimestamp.objects.create(speaker=speaker, timestamp=timezone.now(), instance=self.instance)
        audio = open(os.path.join(self._in_fixtures, 'lamb.mp3'), 'rb')
        recording = Recording.objects.create(audio=File(audio, 'lamb.mp3'), instance=self.instance, start_datetime=timestamp.timestamp)
        recording.timestamps.add(timestamp)
        recording.save()

        files_created = self.helper.split_recording(recording)

        self.assertEqual(len(files_created), 1)
        self.assertSameAudioLength(files_created[0], self.expected_output_file('lamb_whole.mp3'))

    def test_recording_splitting_several_timestamps(self):
        speakers = []
        for i in range(3):
            speakers.append( Speaker.objects.create(name='Steve %d' % i, instance=self.instance) )

        start = timezone.now()
        timestamps = [ 0, 3, 4 ]
        for i in range(3):
            start_i = start + timedelta(seconds=timestamps[i])
            timestamps[i] = RecordingTimestamp.objects.create(speaker=speakers[i], timestamp=start_i, instance=self.instance)

        audio = open(os.path.join(self._in_fixtures, 'lamb.mp3'), 'rb')
        recording = Recording.objects.create(audio=File(audio, 'lamb.mp3'), instance=self.instance, start_datetime=timestamps[0].timestamp)
        for i in range(3):
            recording.timestamps.add(timestamps[i])
        recording.save()

        files_created = self.helper.split_recording(recording)

        self.assertEqual(len(files_created), 3)
        files = [ 'lamb_first_three_seconds.mp3', 'lamb_from_three_to_four_seconds.mp3', 'lamb_from_four_seconds_onwards.mp3' ]
        for i in range(3):
            self.assertSameAudioLength(files_created[i], self.expected_output_file(files[i]))

    def test_audio_length(self):
        audio_path = os.path.join(self._in_fixtures, 'lamb.mp3')
        duration = self.helper.get_audio_duration(audio_path)
        self.assertEqual( duration, 5 )
        



from datetime import datetime

import pyelasticsearch

from django.utils.translation import ugettext_lazy as _
from django.core.management.base import BaseCommand
from django.conf import settings

from haystack.utils import loading

class Command(BaseCommand):
    help = _('Update the default alias to point at search index write alias is using')

    def handle(self, *args, **options):
        connection_options = settings.HAYSTACK_CONNECTIONS['default']

        index_name = connection_options['INDEX_NAME']
        index_write = '%s_write' % index_name

        BackendClass = loading.import_class(connection_options['ENGINE']).backend
        backend = BackendClass('default', **connection_options)

        actions = []
        current_alias = None
        for n, al in backend.conn.aliases().items():
            if index_name in al['aliases']:
                actions.append( { 'remove': { 'index': n, 'alias': index_name } } )
            if index_write in al['aliases']:
                current_alias = n

        if not current_alias:
            raise Exception('There is no "%s" alias to use' % index_write)

        actions.append( { 'add': { 'index': current_alias, 'alias': index_name } } )
        backend.conn.update_aliases({ 'actions': actions })



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Speech'
        db.create_table('speeches_speech', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('created', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('modified', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
            ('audio', self.gf('django.db.models.fields.files.FileField')(max_length=255, blank=True)),
            ('text', self.gf('django.db.models.fields.TextField')(db_index=True, blank=True)),
            ('title', self.gf('django.db.models.fields.TextField')(blank=True)),
            ('event', self.gf('django.db.models.fields.TextField')(db_index=True, blank=True)),
            ('location', self.gf('django.db.models.fields.TextField')(db_index=True, blank=True)),
            ('speaker', self.gf('django.db.models.fields.TextField')(db_index=True, blank=True)),
            ('start', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
            ('end', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
            ('source_url', self.gf('django.db.models.fields.TextField')(blank=True)),
        ))
        db.send_create_signal('speeches', ['Speech'])


    def backwards(self, orm):
        # Deleting model 'Speech'
        db.delete_table('speeches_speech')


    models = {
        'speeches.speech': {
            'Meta': {'object_name': 'Speech'},
            'audio': ('django.db.models.fields.files.FileField', [], {'max_length': '255', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'end': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'event': ('django.db.models.fields.TextField', [], {'db_index': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'location': ('django.db.models.fields.TextField', [], {'db_index': 'True', 'blank': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'source_url': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'speaker': ('django.db.models.fields.TextField', [], {'db_index': 'True', 'blank': 'True'}),
            'start': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'text': ('django.db.models.fields.TextField', [], {'db_index': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.TextField', [], {'blank': 'True'})
        }
    }

    complete_apps = ['speeches']


import datetime
from mock import patch, Mock

from speeches.tests import InstanceTestCase
from speeches.importers.import_akomantoso import ImportAkomaNtoso
from speeches.models import Speech, Speaker, Section
from speeches.importers import import_akomantoso

m = Mock()
m.return_value = open('speeches/fixtures/test_inputs/Debate_Bungeni_1995-10-31.xml', 'rb')

@patch.object(import_akomantoso, 'urlopen', m)
class AkomaNtosoImportTestCase(InstanceTestCase):
    def setUp(self):
        super(AkomaNtosoImportTestCase, self).setUp()
        self.importer = ImportAkomaNtoso(instance=self.instance, commit=True)

    def test_import_sample_file(self):
        self.importer.import_document(
            'speeches/fixtures/test_inputs/Debate_Bungeni_1995-10-31.xml')

        # To get us started, let's just check that we get the right kind of
        # speech in the right order.
        self.assertEqual(
            [x.type for x in Speech.objects.all()],
            [u'scene', u'other', u'narrative', u'speech', u'question',
             u'summary', u'speech', u'answer', u'narrative', u'speech',
             u'narrative']
            )

    def test_import_remote_file(self):
        self.importer.import_document(
            'http://examples.akomantoso.org/php/download.php?file=Debate_Bungeni_1995-10-31.xml')

        # To get us started, let's just check that we get the right kind of
        # speech in the right order.
        self.assertEqual(
            [x.type for x in Speech.objects.all()],
            [u'scene', u'other', u'narrative', u'speech', u'question',
             u'summary', u'speech', u'answer', u'narrative', u'speech',
             u'narrative']
            )

    def test_xpath_preface_elements(self):
        self.importer.import_document(
            'speeches/fixtures/test_inputs/test_xpath.xml')
        self.assertEqual(
            [ x.title for x in Section.objects.all() ],
            [ 'This is the title' ]
        )
        self.assertEqual(
            [ x.start_date for x in Speech.objects.all() ],
            [ datetime.date(2014, 7, 24) ]
        )

    def test_unicode_character(self):
        self.importer.import_document(
            'speeches/fixtures/test_inputs/test_unicode_character.xml')

        self.assertEqual(
            [ x.type for x in Speech.objects.all() ],
            [ 'other' ]
            )

    def test_blank_speakers(self):
        self.importer.import_document(
            'speeches/fixtures/test_inputs/test_blank_speakers.xml')

        speaker = Speaker.objects.get(name='Speaker')
        speeches = Speech.objects.all()
        speeches_s = Speech.objects.filter(type='speech')
        self.assertEqual(speeches.count(), speeches_s.count())

        for i in range(4):
            s = speaker if i%2 else None
            sd = 'Speaker' if i>1 else None
            self.assertEqual(speeches[i].speaker, s)
            self.assertEqual(speeches[i].speaker_display, sd)



import sys
import re
from datetime import date
from mock import patch, Mock
from six.moves.urllib.error import HTTPError

import django
from django.test.utils import override_settings
from django.utils.six import assertRegex
from django.utils.six.moves import builtins
from django.utils.encoding import smart_text
from easy_thumbnails.templatetags import thumbnail

from speeches.tests import InstanceTestCase
from speeches.models import Speaker, Speech, Section
from speeches import models

def side_effect(url):
    if '404' in url:
        raise HTTPError(url, 404, 'NOT FOUND', None, None)
    return ('speeches/fixtures/test_inputs/Ferdinand_Magellan.jpg', None)

@override_settings(MEDIA_URL='/uploads/')
@patch.object(models, 'urlretrieve', Mock(side_effect=side_effect))
class SpeakerTests(InstanceTestCase):
    """Tests for the speaker functionality"""
    speakers = []

    def tearDown(self):
        for speaker in self.speakers:
            speaker.image_cache.delete(save=False)

    def test_speaker_page_lists_speeches(self):
        # Add a speaker
        speaker = Speaker.objects.create(
            name='Steve', instance=self.instance,
            summary='A froody dude',
            image='http://example.com/image.jpg')
        self.speakers.append(speaker)

        # Call the speaker's page
        resp = self.client.get('/speaker/%s' % speaker.slug)

        self.assertSequenceEqual("Steve", resp.context['speaker'].name)
        self.assertSequenceEqual("A froody dude", resp.context['speaker'].summary)
        self.assertSequenceEqual("http://example.com/image.jpg", resp.context['speaker'].image)

        # Assert no speeches
        self.assertSequenceEqual([], resp.context['speech_list'])
        self.assertSequenceEqual([], resp.context['page_obj'])

        # Add a speech
        speech = Speech.objects.create(
            text="A test speech",
            speaker=speaker,
            instance=self.instance,
            start_date=date(2014, 9, 17),
            )

        # Call the speaker's page again
        resp = self.client.get('/speaker/%s' % speaker.slug)

        self.assertSequenceEqual([speech], resp.context['speech_list'])
        self.assertSequenceEqual([speech], resp.context['page_obj'])

        # Add another speech on the same date and check that both dates are displayed
        speech = Speech.objects.create(
            text="Another speech",
            speaker=speaker,
            instance=self.instance,
            start_date=date(2014, 9, 17),
            )

        # Check that a search which returns more than one speech on the same
        # date displays a date for each.
        resp = self.client.get('/speaker/%s' % speaker.slug)
        self.assertEqual(len(re.findall(r'<span class="breadcrumbs__date">\s*17 Sep 2014\s*</span>', resp.content.decode())), 2)

    def test_speaker_page_has_button_to_add_speech(self):
        # Add a speaker
        speaker = Speaker.objects.create(name='Steve', instance=self.instance)

        # Call the speaker's page
        resp = self.client.get('/speaker/%s' % speaker.slug)

        self.assertContains(resp, '<a href="/speech/add?speaker=%d" class="button small right">Add speech</a>' % speaker.id, html=True)


    def test_speaker_headshots_in_speeches_section(self):
        # Test that headshots vs default image work OK
        speaker1 = Speaker.objects.create(
            name='Marilyn',
            instance=self.instance,
            summary='movie star',
            image=u'http://example.com/imag%C3%A9.jpg')
        self.speakers.append(speaker1)
        speaker2 = Speaker.objects.create(name='Salinger',
            instance=self.instance)

        section = Section.objects.create(heading='Test Section',
            instance=self.instance)

        speech1 = Speech.objects.create(
            text="A girl doesn't need anyone that doesn't need her.",
            speaker=speaker1, section=section,
            instance=self.instance, public=True)

        speech2 = Speech.objects.create(
            text="I'm sick of not having the courage to be an absolute nobody.",
            speaker=speaker2, section=section,
            instance=self.instance, public=True)

        resp = self.client.get('/sections/' + str(section.id))

        assertRegex(self, resp.content.decode(), r'(?s)<img src="/uploads/speakers/default/imag%%C3%%A9.jpg.96x96_q85_crop-smart_face_upscale.jpg".*?<a href="/speaker/%s">\s*' % (speaker1.slug))
        assertRegex(self, resp.content.decode(), r'(?s)<img src="\s*/static/speeches/i/a.png\s*".*?<a href="/speaker/%s">\s*' % (speaker2.slug))

    def test_create_speaker_with_long_image_url(self):
        long_image_url = 'http://example.com/image%E2%97%8F123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789.jpg'

        s1 = Speaker.objects.create(
            name='Long Image URL',
            instance=self.instance,
            image=long_image_url,
            )
        s2 = Speaker.objects.create(
            name='Duplicate long image URL',
            instance=self.instance,
            image=long_image_url,
            )
        self.speakers.extend((s1, s2))

        # Note the filename in image_cache has been truncated.
        self.assertEqual(
            s1.image_cache,
            u'speakers/default/image\u25cf12345678901234567890123456789012345678901234567890123456789012345.jpg',
            )

        # The truncated filename for the second speaker has some random stuff at the end.
        # If this get fails it might well mean you need a Django security update
        # https://www.djangoproject.com/weblog/2014/aug/20/security/
        assertRegex(
            self,
            smart_text(s2.image_cache),
            u'^speakers/default/image\u25cf12345678901234567890123456789012345678901234567890123456789012345_.{7}\.jpg$',
            )

    def test_add_speaker_with_whitespace(self):
        name = ' Bob\n'
        self.client.post('/speaker/add', {'name': name})

        speaker = Speaker.objects.order_by('-id')[0]
        self.assertEqual(speaker.name, 'Bob')

    def test_add_speaker_with_image_not_found(self):
        try:
            speaker = Speaker.objects.create(
                name='Not Found',
                instance=self.instance,
                image='http://httpbin.org/status/404')
        except HTTPError:
            self.fail("Speaker unexpectedly raised HTTPError")



SOUTH_ERROR_MESSAGE = """\n
For South support, customize the SOUTH_MIGRATION_MODULES setting like so:

    SOUTH_MIGRATION_MODULES = {
        'speeches': 'speeches.south_migrations',
    }
"""

try:  # Django < 1.7
    from django.db import migrations
except ImportError:
    from django.core.exceptions import ImproperlyConfigured
    raise ImproperlyConfigured(SOUTH_ERROR_MESSAGE)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import DataMigration
from django.db import models

class Migration(DataMigration):

    def forwards(self, orm):
        for speech in orm['speeches.Speech'].objects.all():
            # Take the event field and create a meeting and debate for it
            if speech.event:
                meeting = orm['speeches.Meeting'].objects.create(title=speech.event, date=speech.start_date)
                debate = orm['speeches.Debate'].objects.create(title=speech.event, meeting=meeting)
                speech.debate = debate
                speech.save()

    def backwards(self, orm):
        for speech in orm['speeches.Speech'].objects.all():
            # Put data from a related debate into the event field
            if speech.debate:
                speech.event = speech.debate.title
                speech.debate = None
                speech.save()

        # Delete debates which aren't referenced from any speech
        for debate in orm['speeches.Debate'].objects.all():
            if not orm['speeches.Speech'].objects.all().filter(debate=debate):
                debate.delete()

        # Delete meetings which aren't referenced from any debate
        for meeting in orm['speeches.Meeting'].objects.all():
            if not orm['speeches.Debate'].objects.all().filter(meeting=meeting):
                meeting.delete();

    models = {
        'speeches.debate': {
            'Meta': {'object_name': 'Debate'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'meeting': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['speeches.Meeting']", 'null': 'True', 'blank': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'speeches.meeting': {
            'Meta': {'object_name': 'Meeting'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'speeches.speaker': {
            'Meta': {'object_name': 'Speaker'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.TextField', [], {'db_index': 'True'}),
            'popit_url': ('django.db.models.fields.TextField', [], {'unique': 'True'})
        },
        'speeches.speech': {
            'Meta': {'object_name': 'Speech'},
            'audio': ('django.db.models.fields.files.FileField', [], {'max_length': '255', 'blank': 'True'}),
            'celery_task_id': ('django.db.models.fields.CharField', [], {'max_length': '256', 'null': 'True', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'debate': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['speeches.Debate']", 'null': 'True', 'blank': 'True'}),
            'end_date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'end_time': ('django.db.models.fields.TimeField', [], {'null': 'True', 'blank': 'True'}),
            'event': ('django.db.models.fields.TextField', [], {'db_index': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'location': ('django.db.models.fields.TextField', [], {'db_index': 'True', 'blank': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'source_url': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'speaker': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['speeches.Speaker']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'}),
            'start_date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'start_time': ('django.db.models.fields.TimeField', [], {'null': 'True', 'blank': 'True'}),
            'text': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'title': ('django.db.models.fields.TextField', [], {'blank': 'True'})
        }
    }

    complete_apps = ['speeches']
    symmetrical = True



# -*- coding: utf-8 -*-

import logging

from dateutil import parser as dateutil
from lxml import etree
from lxml import objectify
from six.moves.urllib.request import urlopen

from speeches.importers.import_base import ImporterBase
from speeches.models import Section, Speech, Speaker

logger = logging.getLogger(__name__)


class ImportAkomaNtoso (ImporterBase):
    start_date = None

    def import_document(self, document_path):
        if document_path.startswith('http'):
            self.xml = objectify.fromstring(urlopen(document_path).read())
        else:
            self.xml = objectify.parse(document_path).getroot()
        self.ns = self.xml.nsmap.get(None, None)
        return self.parse_document()

    def parse_document(self):
        debate = self.xml.debate

        if self.ns:
            people = debate.findall(
                'an:meta/an:references/an:TLCPerson',
                namespaces={'an': self.ns},
                )
        else:
            people = debate.findall('meta/references/TLCPerson')
        if people is None:
            people = []
        for person in people:
            id = person.get('id')
            href = person.get('href')
            try:
                speaker = Speaker.objects.get(
                    instance=self.instance, identifiers__identifier=href)
            except Speaker.DoesNotExist:
                speaker = Speaker(
                    instance=self.instance, name=person.get('showAs'))
                if self.commit:
                    speaker.save()
                    speaker.identifiers.create(
                        identifier=href, scheme='Akoma Ntoso import')

            self.speakers[id] = speaker

        docDate = self.get_preface_tag(debate, 'docDate')
        if docDate:
            self.start_date = dateutil.parse(docDate.get('date'))

        docTitle = self.get_preface_tag(debate, 'docTitle')
        if docTitle:
            docTitle = docTitle.text

        docNumber = self.get_preface_tag(debate, 'docNumber')
        if docNumber:
            docNumber = docNumber.text

        legislature = self.get_preface_tag(debate, 'legislature')
        if legislature:
            legislature = legislature.text

        session = self.get_preface_tag(debate, 'session')
        if session:
            session = session.text

        section = None
        if docTitle:
            section = self.make(
                Section,
                parent=None,
                heading=docTitle,
                start_date=self.start_date,
                number=docNumber or '',
                legislature=legislature or '',
                session=session or '',
            )

        self.visit(debate.debateBody, section)

    def get_preface_tag(self, debate, tag):
        if self.ns:
            tag = debate.xpath('an:coverPage//an:%s|an:preface//an:%s' % (tag, tag), namespaces={'an': self.ns})
        else:
            tag = debate.xpath('coverPage//%s|preface//%s' % (tag, tag))
        if tag:
            return tag[0]

    def get_tag(self, node):
        return etree.QName(node.tag).localname

    def get_text(self, node):
        paras = [node.text]
        paras += [
            etree.tostring(child, encoding='utf-8').decode('utf-8')
            for child in node.iterchildren()
            if self.get_tag(child) not in ('num', 'heading', 'subheading', 'from')
            ]
        return ''.join(filter(None, paras))

    def construct_heading(self, node):
        headings = {}
        for tag in ('num', 'heading', 'subheading'):
            if hasattr(node, tag):
                headings[tag] = getattr(node, tag).text
        return headings

    def construct_datetime(self, time):
        if not time:
            return (None, None)
        dt = dateutil.parse(time)
        return dt.date(), dt.time()

    def get_speaker(self, child):
        if hasattr(child, 'from'):
            display_name = child['from'].text
        else:
            display_name = None

        by_ref = child.get('by')
        if by_ref:
            if not by_ref.startswith('#'):
                logger.warn(
                    "by attribute value doesn't begin with '#': %s" % by_ref)
            speaker = self.speakers[by_ref[1:]]
        else:
            speaker = None

        return speaker, display_name

    def handle_tag(self, node, section):
        """If we need to do something out of the ordinary handling elements,
        subclass it here"""
        return False

    def visit(self, node, section):
        for child in node.iterchildren():
            tagname = self.get_tag(child)
            if tagname in ('num', 'heading', 'subheading'):
                # this will already have been extracted
                continue
            if tagname in (
                    'debateSection', 'administrationOfOath', 'rollCall',
                    'prayers', 'oralStatements', 'writtenStatements',
                    'personalStatements', 'ministerialStatements',
                    'resolutions', 'nationalInterest', 'declarationOfVote',
                    'communication', 'petitions', 'papers', 'noticesOfMotion',
                    'questions', 'address', 'proceduralMotions',
                    'pointOfOrder', 'adjournment',
                    ):
                headings = self.construct_heading(child)
                childSection = self.make(Section,
                    parent=section,
                    start_date=self.start_date,
                    **headings
                )
                self.visit(child, childSection)
            elif tagname in ('speech', 'question', 'answer'):
                headings = self.construct_heading(child)
                text = self.get_text(child)
                start_date, start_time = self.construct_datetime(child.get('startTime'))
                end_date, end_time = self.construct_datetime(child.get('endTime'))
                speaker, display_name = self.get_speaker(child)
                self.make(
                    Speech,
                    section=section,
                    start_date=start_date or self.start_date,
                    start_time=start_time,
                    end_date=end_date,
                    end_time=end_time,
                    text=text,
                    speaker=speaker,
                    speaker_display=display_name,
                    type=tagname,
                    **headings
                    )
            elif tagname in ('scene', 'narrative', 'summary', 'other'):
                text = self.get_text(child)

                self.make(
                    Speech,
                    section=section,
                    start_date=self.start_date,
                    text=text,
                    type=tagname,
                    )
            else:
                success = self.handle_tag(child, section)
                if not success:
                    logger.error(
                        '%s unrecognised, "%s" - %s' %
                        (child.tag, child, self.get_text(child))
                        )



from django import template
from django.template.defaultfilters import stringfilter
from django.utils.html import linebreaks
from django.utils.safestring import mark_safe, SafeData

import bleach
from django_bleach.templatetags.bleach_tags import bleach_args

register = template.Library()

@register.filter(needs_autoescape=True)
@stringfilter
def linebreaks_with_lead(value, autoescape=None):
    autoescape = autoescape and not isinstance(value, SafeData)
    out = linebreaks(value, autoescape)
    out = out.replace('<p>', '<p class="lead">', 1)
    return mark_safe(out)

@register.filter()
def striptags_highlight(value):
    bleached_value = bleach.clean(value, tags=['em'], strip=True)
    return mark_safe(bleached_value)

@register.filter('bleach')
def bleach_value(value):
    """Same as django_bleach, but convert the <br> we get back to valid XML,
    for the AN export."""
    bleached_value = bleach.clean(value, **bleach_args)
    bleached_value = bleached_value.replace('<br>', '<br/>')
    return mark_safe(bleached_value)



import datetime
import re

from django.core.management import call_command
from haystack.query import SearchQuerySet

from speeches.models import Speech, Speaker, Section
from speeches.tests import InstanceTestCase

class SearchTests(InstanceTestCase):
    def test_search(self):
        s1 = self.instance.speaker_set.create(name='Speaker 1')
        s2 = self.instance.speaker_set.create(name='Speaker 2')
        self.instance.speech_set.create(speaker=s1, text='Some text by speaker 1')
        self.instance.speech_set.create(speaker=s1, text='Some more text by speaker 1')
        self.instance.speech_set.create(speaker=s2, text='Some text by speaker 2')
        call_command('rebuild_index', verbosity=0, interactive=False)

        results = SearchQuerySet().models(Speech)
        self.assertEqual( results.count(), 3 )
        self.assertEqual( results.filter(speaker=s1.id).count(), 2 )
        self.assertEqual( results.filter(speaker=s2.id).count(), 1 )
        self.assertEqual( results.filter(instance=None).count(), 0 )

    def test_search_results(self):
        s1 = self.instance.speaker_set.create(name='Speaker 1')
        s2 = self.instance.speaker_set.create(name='Speaker 2')
        self.instance.speech_set.create(
            speaker=s1,
            text='Some text by speaker 1',
            start_date=datetime.date(2014, 9, 17),
            )
        self.instance.speech_set.create(
            speaker=s1,
            text='Some more text by speaker 1',
            start_date=datetime.date(2014, 9, 17),
            )
        self.instance.speech_set.create(speaker=s2, text='Some text by speaker 2')
        call_command('rebuild_index', verbosity=0, interactive=False)

        resp = self.client.get('/search/?q=more')
        self.assertContains(resp, 'Some <em>more</em> text by speaker 1')

        # Check that a search which returns more than one speech on the same
        # date displays a date for each.
        resp = self.client.get('/search/?q="speaker 1"')
        assert len(re.findall(r'<span class="speech__meta-data__date">\s*17 Sep 2014\s*</span>', resp.content.decode())) == 2



import datetime
from haystack import indexes
from speeches.models import Speech, Speaker, Section

class SpeechIndex(indexes.SearchIndex, indexes.Indexable):
    # Use a template here to include speaker name as well... TODO
    text = indexes.CharField(document=True, model_attr='text') # , use_template=True)
    title = indexes.CharField(model_attr='heading') # use_template=True)
    start_date = indexes.DateTimeField(model_attr='start_date', null=True)
    instance = indexes.CharField(model_attr='instance__label')
    speaker = indexes.IntegerField(model_attr='speaker_id', null=True)

    def get_model(self):
        return Speech

    def index_queryset(self, using=None):
        return self.get_model()._default_manager.select_related('instance')

    def get_updated_field(self):
        return 'modified'

class SpeakerIndex(indexes.SearchIndex, indexes.Indexable):
    text = indexes.CharField(document=True, model_attr='name')
    instance = indexes.CharField(model_attr='instance__label')

    def get_model(self):
        return Speaker

    def index_queryset(self, using=None):
        return self.get_model()._default_manager.select_related('instance')

    def get_updated_field(self):
        return 'updated_at'

class SectionIndex(indexes.SearchIndex, indexes.Indexable):
    text = indexes.CharField(document=True, model_attr='heading')
    instance = indexes.CharField(model_attr='instance__label')

    def get_model(self):
        return Section

    def index_queryset(self, using=None):
        return self.get_model()._default_manager.select_related('instance')

    def get_updated_field(self):
        return 'modified'



from nose.plugins import Plugin

class SkipMigrations(Plugin):
    name = 'skip-migrations'
    enabled = True

    def configure(self, options, conf):
        pass

    def wantDirectory(self, dirname):
        if 'migrations' in dirname:
            return False
        return None



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import DataMigration
from django.db import models

class Migration(DataMigration):

    def forwards(self, orm):
        for speech in orm['speeches.Speech'].objects.all():
            # Take the datetime fields and divide them into a date and time
            if speech.start:
                speech.start_date = speech.start.date()
                speech.start_time = speech.start.time()

            if speech.end:
                speech.end_date = speech.end.date()
                speech.end_time = speech.end.time()

            speech.save()


    def backwards(self, orm):
        for speech in orm['speeches.Speech'].objects.all():
            # Take the date/time fields and combine them into a datetime
            if speech.start_date:
                if speech.start_time:
                    speech.start = datetime.datetime.combine(speech.start_date, speech.start_time)
                else:
                    # Explicitly set time to midnight - this will complain about timezones being
                    # naive, but for the data this will migrate we don't care
                    speech.start = datetime.datetime.combine(speech.start_date, datetime.time.min)

            if speech.end_date:
                if speech.end_time:
                    speech.end = datetime.datetime.combine(speech.end_date, speech.end_time)
                else:
                    # Explicitly set time to midnight - this will complain about timezones being
                    # naive, but for the data this will migrate we don't care
                    speech.end = datetime.datetime.combine(speech.end_date, datetime.time.min)

            speech.save()

    models = {
        'speeches.speaker': {
            'Meta': {'object_name': 'Speaker'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.TextField', [], {'db_index': 'True'}),
            'popit_url': ('django.db.models.fields.TextField', [], {'unique': 'True'})
        },
        'speeches.speech': {
            'Meta': {'object_name': 'Speech'},
            'audio': ('django.db.models.fields.files.FileField', [], {'max_length': '255', 'blank': 'True'}),
            'celery_task_id': ('django.db.models.fields.CharField', [], {'max_length': '256', 'null': 'True', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'end': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'end_date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'end_time': ('django.db.models.fields.TimeField', [], {'null': 'True', 'blank': 'True'}),
            'event': ('django.db.models.fields.TextField', [], {'db_index': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'location': ('django.db.models.fields.TextField', [], {'db_index': 'True', 'blank': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'source_url': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'speaker': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['speeches.Speaker']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'}),
            'start': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'start_date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'start_time': ('django.db.models.fields.TimeField', [], {'null': 'True', 'blank': 'True'}),
            'text': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'title': ('django.db.models.fields.TextField', [], {'blank': 'True'})
        }
    }

    complete_apps = ['speeches']
    symmetrical = True



BLEACH_ALLOWED_TAGS = [
    'a', 'abbr', 'b', 'i', 'u', 'span', 'sub', 'sup', 'br',
    'p',
    'ol', 'ul', 'li',
    'table', 'caption', 'tr', 'th', 'td',
]

BLEACH_ALLOWED_ATTRIBUTES = {
    '*': [ 'id', 'title' ], # class, style
    'a': [ 'href' ],
    'li': [ 'value' ],
}



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        pass


    def backwards(self, orm):
        pass


    models = {
        
    }

    complete_apps = ['speeches']



import os
from optparse import make_option

from django.core.management.base import BaseCommand, CommandError

from instances.models import Instance

class ImportCommand(BaseCommand):

    importer_class = None
    document_extension = ''

    option_list = BaseCommand.option_list + (
        make_option('--commit', action='store_true', help='Whether to commit to the database or not'),
        make_option('--instance', action='store', help='Label of instance to add data to'),
        make_option('--file', action='store', help='document to import'),
        make_option('--dir',  action='store', help='directory of documents to import'),
        make_option('--start-date',  action='store', default='', help='earliest date to process, in yyyy-mm-dd format'),
        make_option('--dump-users',  action='store', default='', help='dump a json list to <file> (only valid with --dir for now)'),
    )

    def handle(self, *args, **options):
        verbosity = int(options['verbosity'])

        if options['commit']:
            if not options['instance']:
                raise CommandError("You must specify an instance")
            instance, _ = Instance.objects.get_or_create(label=options['instance'])
        else:
            instance = Instance(label=options['instance'])
        options['instance'] = instance

        if options['file']:
            filename = os.path.expanduser(options['file'])
            (section, speakers) = self.import_document(filename, **options)
            if verbosity > 1:
                if section and section.id:
                    self.stdout.write("Imported section %d\n\n" % section.id)
        elif options['dir']:
            files = self.document_list(options)

            if not len(files):
                raise CommandError("No .%s files found in directory" % self.document_extension)

            imports = [self.import_document(f, **options) for f in files]

            if options['commit']:
                sections = [a for a,_ in imports]
                if verbosity > 1:
                    self.stdout.write("Imported sections %s\n\n"
                        % str( [s.id for s in sections]))

            dump_users = os.path.expanduser(options['dump_users'])
            if dump_users:
                speakers = {}
                for (_,d) in imports:
                    speakers.update(d)

                out = open(dump_users, 'w')
                speakers_list = [ (k, speakers[k]) for k in speakers]
                out.write( json.dumps( speakers_list, indent=4 ) )

                if verbosity > 1:
                    self.stdout.write("Saved speakers list to %s\n" % dump_users)

        else:
            self.stdout.write( self.help )

    def document_list(self, options):
        dir = os.path.expanduser(options['dir'])

        start_date = options['start_date']
        valid = lambda f: f >= start_date if start_date else lambda _: True

        return [ os.path.join(root, filename)
                for (root, _, files)
                in os.walk(dir)
                for filename in files
                if filename[-4:] == '.%s' % self.document_extension
                and valid(filename)]

    def document_valid(self, path):
        return os.path.isfile(path)

    def import_document(self, path, **options):
        verbosity = int(options['verbosity'])

        if not self.document_valid(path):
            raise CommandError("No document found")

        if verbosity > 1:
            self.stdout.write("Starting import: %s\n" % path)

        if self.importer_class == None:
            raise CommandError("No importer_class specified!")

        importer = self.importer_class(**options)

        try:
            section = importer.import_document(path)
        except Exception as e:
            self.stderr.write(str(e))
            return (None, {})

        return (section, importer.speakers)





# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'Speaker.image_cache'
        db.add_column(u'speeches_speaker', 'image_cache',
                      self.gf('django.db.models.fields.files.ImageField')(max_length=100, null=True, blank=True),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'Speaker.image_cache'
        db.delete_column(u'speeches_speaker', 'image_cache')


    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Group']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Permission']"}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'instances.instance': {
            'Meta': {'object_name': 'Instance'},
            'created_by': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'created_instances'", 'null': 'True', 'to': u"orm['auth.User']"}),
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('instances.fields.DNSLabelField', [], {'unique': 'True', 'max_length': '63', 'db_index': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'users': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'instances'", 'blank': 'True', 'to': u"orm['auth.User']"})
        },
        u'popolo.person': {
            'Meta': {'object_name': 'Person'},
            'additional_name': ('django.db.models.fields.CharField', [], {'max_length': '128', 'blank': 'True'}),
            'biography': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'birth_date': ('django.db.models.fields.CharField', [], {'max_length': '10', 'blank': 'True'}),
            'created_at': ('model_utils.fields.AutoCreatedField', [], {'default': 'datetime.datetime.now'}),
            'death_date': ('django.db.models.fields.CharField', [], {'max_length': '10', 'blank': 'True'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'null': 'True', 'blank': 'True'}),
            'end_date': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True', 'blank': 'True'}),
            'family_name': ('django.db.models.fields.CharField', [], {'max_length': '128', 'blank': 'True'}),
            'gender': ('django.db.models.fields.CharField', [], {'max_length': '128', 'blank': 'True'}),
            'given_name': ('django.db.models.fields.CharField', [], {'max_length': '128', 'blank': 'True'}),
            'honorific_prefix': ('django.db.models.fields.CharField', [], {'max_length': '128', 'blank': 'True'}),
            'honorific_suffix': ('django.db.models.fields.CharField', [], {'max_length': '128', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.URLField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'patronymic_name': ('django.db.models.fields.CharField', [], {'max_length': '128', 'blank': 'True'}),
            'sort_name': ('django.db.models.fields.CharField', [], {'max_length': '128', 'blank': 'True'}),
            'start_date': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True', 'blank': 'True'}),
            'summary': ('django.db.models.fields.CharField', [], {'max_length': '512', 'blank': 'True'}),
            'updated_at': ('model_utils.fields.AutoLastModifiedField', [], {'default': 'datetime.datetime.now'})
        },
        u'speeches.recording': {
            'Meta': {'object_name': 'Recording'},
            'audio': ('django.db.models.fields.files.FileField', [], {'max_length': '255'}),
            'audio_duration': ('django.db.models.fields.IntegerField', [], {'default': '0', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'instance': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['instances.Instance']"}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'start_datetime': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'})
        },
        u'speeches.recordingtimestamp': {
            'Meta': {'ordering': "('timestamp',)", 'object_name': 'RecordingTimestamp'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'instance': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['instances.Instance']"}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'recording': ('django.db.models.fields.related.ForeignKey', [], {'default': '0', 'related_name': "'timestamps'", 'to': u"orm['speeches.Recording']"}),
            'speaker': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['speeches.Speaker']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'}),
            'speech': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['speeches.Speech']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'}),
            'timestamp': ('django.db.models.fields.DateTimeField', [], {'db_index': 'True'})
        },
        u'speeches.section': {
            'Meta': {'ordering': "('id',)", 'unique_together': "(('parent', 'slug', 'instance'),)", 'object_name': 'Section'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'heading': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'instance': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['instances.Instance']"}),
            'legislature': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'num': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'number': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'children'", 'null': 'True', 'to': u"orm['speeches.Section']"}),
            'session': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'slug': ('sluggable.fields.SluggableField', [], {'unique_with': "('parent', 'instance')", 'max_length': '50', 'populate_from': "'title'"}),
            'source_url': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'start_date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'start_time': ('django.db.models.fields.TimeField', [], {'null': 'True', 'blank': 'True'}),
            'subheading': ('django.db.models.fields.TextField', [], {'blank': 'True'})
        },
        u'speeches.slug': {
            'Meta': {'object_name': 'Slug'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'redirect': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'slug': ('django.db.models.fields.CharField', [], {'max_length': '255', 'db_index': 'True'})
        },
        u'speeches.speaker': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('instance', 'slug'),)", 'object_name': 'Speaker', '_ormbases': [u'popolo.Person']},
            'image_cache': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'instance': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['instances.Instance']"}),
            u'person_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': u"orm['popolo.Person']", 'unique': 'True', 'primary_key': 'True'}),
            'slug': ('sluggable.fields.SluggableField', [], {'unique_with': "('instance',)", 'max_length': '50', 'populate_from': "'name'"})
        },
        u'speeches.speech': {
            'Meta': {'ordering': "('start_date', 'start_time', 'id')", 'object_name': 'Speech'},
            'audio': ('django.db.models.fields.files.FileField', [], {'max_length': '255', 'blank': 'True'}),
            'celery_task_id': ('django.db.models.fields.CharField', [], {'max_length': '256', 'null': 'True', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'end_date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'end_time': ('django.db.models.fields.TimeField', [], {'null': 'True', 'blank': 'True'}),
            'event': ('django.db.models.fields.TextField', [], {'db_index': 'True', 'blank': 'True'}),
            'heading': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'instance': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['instances.Instance']"}),
            'location': ('django.db.models.fields.TextField', [], {'db_index': 'True', 'blank': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'num': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'public': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'section': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['speeches.Section']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'}),
            'source_url': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'speaker': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['speeches.Speaker']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'}),
            'speaker_display': ('django.db.models.fields.CharField', [], {'max_length': '256', 'null': 'True', 'blank': 'True'}),
            'start_date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'start_time': ('django.db.models.fields.TimeField', [], {'null': 'True', 'blank': 'True'}),
            'subheading': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'tags': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'to': u"orm['speeches.Tag']", 'null': 'True', 'blank': 'True'}),
            'text': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'type': ('django.db.models.fields.CharField', [], {'max_length': '32'})
        },
        u'speeches.tag': {
            'Meta': {'object_name': 'Tag'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'instance': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['instances.Instance']"}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'})
        }
    }

    complete_apps = ['speeches']


import os
import tempfile
import shutil

from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC

from django.test.utils import override_settings
from django.utils import unittest
from django.conf import settings

import speeches
from speeches.models import Speaker, Speech
from speeches.tests import InstanceLiveServerTestCase

skip_selenium = not os.environ.get('SELENIUM_TESTS', False)

@unittest.skipIf(skip_selenium, 'Selenium tests not requested')
@override_settings(MEDIA_ROOT=tempfile.mkdtemp(), ATT_API_URL='http://att.api.url.example.org/')
class SeleniumTests(InstanceLiveServerTestCase):
    @classmethod
    def setUpClass(cls):
        if getattr(cls, "__unittest_skip__", False):
            return
        cls.selenium = webdriver.Firefox()
        cls.selenium.implicitly_wait(5)
        cls._in_fixtures = os.path.join(os.path.abspath(speeches.__path__[0]), 'fixtures', 'test_inputs')
        super(SeleniumTests, cls).setUpClass()

    @classmethod
    def tearDownClass(cls):
        if getattr(cls, "__unittest_skip__", False):
            return
        cls.selenium.quit()
        super(SeleniumTests, cls).tearDownClass()

    def tearDown(self):
        # Clear the speeches folder if it exists
        speeches_folder = os.path.join(settings.MEDIA_ROOT, 'speeches')
        if(os.path.exists(speeches_folder)):
            shutil.rmtree(speeches_folder)

    @property
    def live_server_url(self):
        url = super(SeleniumTests, self).live_server_url
        return url.replace('localhost', 'testing.127.0.0.1.xip.io')

    def test_select_text(self):
        self.selenium.get('%s%s' % (self.live_server_url, '/speech/add'))

        speech_form = WebDriverWait(self.selenium, 10).until(
            EC.visibility_of_element_located((By.ID, 'speech-form'))
        )
        self.assertIsNotNone(speech_form)
        self.assertTrue(self.selenium.find_element_by_id("id_text_controls").is_displayed())

    def test_add_speech(self):
        self.selenium.get('%s%s' % (self.live_server_url, '/speech/add'))
        text_input = WebDriverWait(self.selenium, 10).until(
            EC.presence_of_element_located((By.NAME, 'text'))
        )
        text_input.send_keys('This is a speech')
        self.selenium.find_element_by_xpath('//input[@value="Save speech"]').click()
        WebDriverWait(self.selenium, 10).until(
            EC.title_contains('This is a speech')
        )
        speech = Speech.objects.order_by('-created')[0]
        self.assertIn('/speech/%d' % speech.id, self.selenium.current_url)

    def test_speaker_autocomplete(self):
        # Put a person in the db for the autocomplete to find
        speaker = Speaker.objects.create(name='Name', instance=self.instance)

        # Type a name in
        self.selenium.get('%s%s' % (self.live_server_url, '/speech/add'))
        speaker_input = WebDriverWait(self.selenium, 10).until(
            EC.element_to_be_clickable((By.XPATH, "//div[@id='s2id_id_speaker']/child::a"))
        )
        speaker_input.click()
        text_input = self.selenium.find_element_by_xpath('//div[@id="select2-drop"]//input')
        text_input.send_keys('Na')
        # Wait for autocomplete
        WebDriverWait(self.selenium, 10).until(
            lambda x: x.find_element_by_xpath('//div[@class="select2-result-label"]')
        )
        # First result is what was typed (for creating new entry), so click second
        self.selenium.find_element_by_xpath('(//div[@class="select2-result-label"])[2]').click()
        # Check it is selected
        selection_element = self.selenium.find_element_by_xpath('//div[@id="s2id_id_speaker"]/descendant::span')
        self.assertIn('Name', selection_element.text)
        speaker_input = self.selenium.find_element_by_id("id_speaker")
        self.assertTrue(speaker_input.get_attribute('value') == str(speaker.id))
        # Check we can unselect it
        self.selenium.find_element_by_xpath('//div[@id="s2id_id_speaker"]/descendant::abbr[@class="select2-search-choice-close"]').click()
        speaker_input = self.selenium.find_element_by_id("id_speaker")
        self.assertTrue(speaker_input.get_attribute('value') == "")



from six.moves.urllib.parse import urlencode
import json

from django_select2.util import register_field

from instances.models import Instance

from speeches.tests import InstanceTestCase

from speeches.models import Speaker, Section
from speeches.forms import SpeakerField, SectionField


class AjaxTests(InstanceTestCase):
    def setUp(self, *args, **kwargs):
        super(AjaxTests, self).setUp(*args, **kwargs)

        Speaker.objects.create(name='Alice', instance=self.instance)
        Speaker.objects.create(name='Alastair', instance=self.instance)
        Speaker.objects.create(name='Bob', instance=self.instance)

        Section.objects.create(heading='Section A', instance=self.instance)
        Section.objects.create(heading='Section B', instance=self.instance)
        Section.objects.create(heading='Not This', instance=self.instance)

        other_instance = Instance.objects.create(label='other')
        Speaker.objects.create(name='Alan', instance=other_instance)
        Section.objects.create(heading='Other', instance=other_instance)

    def test_lookup_speaker(self):
        # Copy what happens in AutoViewFieldMixin's __init__
        # in order to get the required field_id.
        field_id = register_field('speeches.forms.SpeakerField', SpeakerField())

        # The ajax queries look something like this:
        # /select2/fields/auto.json?term=al&page=1&context=&field_id=f5af12d0dbb3800ea6b8d88b4720ad7b625f1ae4&_=1399984568706
        data = urlencode({
            'term': 'al',
            'field_id': field_id,
            'page': 1,
            'context': '',
            })
        resp = self.client.get('/select2/fields/auto.json?' + data)

        results = json.loads(resp.content.decode())['results']

        # We should see Alice and Alastair, but not Bob (doesn't match),
        # or Alan (wrong instance).
        self.assertEqual(
            set([x['text'] for x in results]),
            set((u'Alice', u'Alastair'))
            )

    def test_lookup_section(self):
        field_id = register_field('speeches.forms.SectionField', SectionField())
        data = urlencode({
            'term': 'se',
            'field_id': field_id,
            'page': 1,
            'context': '',
        })
        resp = self.client.get('/select2/fields/auto.json?' + data)

        results = json.loads(resp.content.decode())['results']

        # We should see Sections, but not Not This (doesn't match),
        # or Other (wrong instance).
        self.assertEqual(
            set([x['text'] for x in results]),
            set((u'Section A', u'Section B'))
        )



# -*- coding: utf-8 -*-

from speeches.tests import InstanceTestCase
from speeches.models import Speaker, Section

class SlugTests(InstanceTestCase):

    def test_all_latin_speaker_slug(self):
        latin_speaker = Speaker.objects.create(
            name='Foo Bar',
            instance=self.instance
        )
        try:
            self.assertEqual(
                latin_speaker.slug,
                'foo-bar'
            )
        finally:
            latin_speaker.delete()

    def test_all_cjk_speaker_slug(self):
        cjk_speaker = Speaker.objects.create(
            name=u'張家祝',
            instance=self.instance
        )
        cjk_speaker_duplicate = Speaker.objects.create(
            name=u'張家祝',
            instance=self.instance
        )
        try:
            self.assertEqual(
                cjk_speaker.slug,
                u'張家祝',
            )
            self.assertEqual(
                cjk_speaker_duplicate.slug,
                u'張家祝-2',
            )
        finally:
            cjk_speaker.delete()
            cjk_speaker_duplicate.delete()

    def test_cyrillic_speaker_slug(self):
        cyrillic_speaker = Speaker.objects.create(
            name=u'Борщ со сметаной',
            instance=self.instance
        )
        try:
            self.assertEqual(
                cyrillic_speaker.slug,
                u'борщ-со-сметаной',
            )
        finally:
            cyrillic_speaker.delete()

    def test_all_cjk_section_slug(self):
        cjk_section = Section.objects.create(
            heading=u'經貿國是會議全國大會總結',
            instance=self.instance
        )
        try:
            self.assertEqual(
                cjk_section.slug,
                u'經貿國是會議全國大會總結',
            )
        finally:
            cjk_section.delete()



from datetime import datetime

import pyelasticsearch

from django.utils.translation import ugettext_lazy as _
from django.core.management.base import BaseCommand
from django.conf import settings

from haystack.utils import loading

class Command(BaseCommand):
    help = _('Create a new dated search index for reindexing with no downtime')

    def handle(self, *args, **options):
        connection_options = settings.HAYSTACK_CONNECTIONS['default']

        index_name = connection_options['INDEX_NAME']
        new_alias = '%s_%s' % (index_name, datetime.now().isoformat().lower())
        connection_options['INDEX_NAME'] = new_alias

        BackendClass = loading.import_class(connection_options['ENGINE']).backend
        backend = BackendClass('default', **connection_options)
        backend.setup()

        index_write = '%s_write' % index_name

        aliases = backend.conn.aliases()
        actions = []
        for n, al in aliases.items():
            if index_write in al['aliases']:
                actions.append( { 'remove': { 'index': n, 'alias': index_write } } )

        actions.append( { 'add': { 'index': new_alias, 'alias': index_write } } )
        backend.conn.update_aliases({ 'actions': actions })
